@@ -25,13 +25,12 @@ import sys
 import glob
 import subprocess
 import shutil
-import netifaces
 import time
 
 from .. import utils
 from ...configmanager import ConfigManager, ConfigurationError
 from ..sriov import apply_sriov_config
-from ..ovs import OvsDbServerNotRunning, apply_ovs_cleanup
+from ..ovs import OvsDbServerNotRunning, OvsDbServerNotInstalled, apply_ovs_cleanup
 
 
 OVS_CLEANUP_SERVICE = 'netplan-ovs-cleanup.service'
@@ -116,7 +115,7 @@ class NetplanApply(utils.NetplanCommand):
             old_ovs_glob.remove(ovs_cleanup_service)
         old_files_ovs = bool(old_ovs_glob)
         old_nm_glob = glob.glob('/run/NetworkManager/system-connections/netplan-*')
-        nm_ifaces = utils.nm_interfaces(old_nm_glob, netifaces.interfaces())
+        nm_ifaces = utils.nm_interfaces(old_nm_glob, utils.get_interfaces())
         old_files_nm = bool(old_nm_glob)
 
         generator_call = []
@@ -132,7 +131,7 @@ class NetplanApply(utils.NetplanCommand):
             else:
                 raise ConfigurationError("the configuration could not be generated")
 
-        devices = netifaces.interfaces()
+        devices = utils.get_interfaces()
 
         # Re-start service when
         # 1. We have configuration files for it
@@ -168,17 +167,16 @@ class NetplanApply(utils.NetplanCommand):
             # Clean up any old netplan related OVS ports/bonds/bridges, if applicable
             NetplanApply.process_ovs_cleanup(config_manager, old_files_ovs, restart_ovs, exit_on_error)
             wpa_services = ['netplan-wpa-*.service']
-            # Historically (up to v0.98) we had netplan-wpa@*.service files, in case of an
-            # upgraded system, we need to make sure to stop those.
-            if utils.systemctl_is_active('netplan-wpa@*.service'):
-                wpa_services.insert(0, 'netplan-wpa@*.service')
             utils.systemctl('stop', wpa_services, sync=sync)
         else:
             logging.debug('no netplan generated networkd configuration exists')
 
+        loopback_connection = ''
         if restart_nm:
             logging.debug('netplan generated NM configuration changed, restarting NM')
             if utils.nm_running():
+                if 'lo' in nm_ifaces:
+                    loopback_connection = utils.nm_get_connection_for_interface('lo')
                 # restarting NM does not cause new config to be applied, need to shut down devices first
                 for device in devices:
                     if device not in nm_ifaces:
@@ -194,7 +192,7 @@ class NetplanApply(utils.NetplanCommand):
             logging.debug('no netplan generated NM configuration exists')
 
         # Refresh devices now; restarting a backend might have made something appear.
-        devices = netifaces.interfaces()
+        devices = utils.get_interfaces()
 
         # evaluate config for extra steps we need to take (like renaming)
         # for now, only applies to non-virtual (real) devices.
@@ -214,7 +212,7 @@ class NetplanApply(utils.NetplanCommand):
         # the interface name, if it was already renamed once (e.g. during boot),
         # because of the NamePolicy=keep default:
         # https://www.freedesktop.org/software/systemd/man/systemd.net-naming-scheme.html
-        devices = netifaces.interfaces()
+        devices = utils.get_interfaces()
         for device in devices:
             logging.debug('netplan triggering .link rules for %s', device)
             try:
@@ -230,7 +228,7 @@ class NetplanApply(utils.NetplanCommand):
             except subprocess.CalledProcessError:
                 logging.debug('Ignoring device without syspath: %s', device)
 
-        devices_after_udev = netifaces.interfaces()
+        devices_after_udev = utils.get_interfaces()
         # apply some more changes manually
         for iface, settings in changes.items():
             # rename non-critical network interfaces
@@ -253,10 +251,8 @@ class NetplanApply(utils.NetplanCommand):
                                       stdout=subprocess.DEVNULL,
                                       stderr=subprocess.DEVNULL)
 
-        # Reloading of udev rules happens during 'netplan generate' already
-        # subprocess.check_call(['udevadm', 'control', '--reload-rules'])
-        subprocess.check_call(['udevadm', 'trigger', '--attr-match=subsystem=net'])
-        subprocess.check_call(['udevadm', 'settle'])
+        subprocess.check_call(['udevadm', 'control', '--reload'])
+        subprocess.check_call(['udevadm', 'trigger', '--action=move', '--subsystem-match=net', '--settle'])
 
         # apply any SR-IOV related changes, if applicable
         NetplanApply.process_sriov_config(config_manager, exit_on_error)
@@ -286,12 +282,20 @@ class NetplanApply(utils.NetplanCommand):
         if restart_nm:
             # Flush all IP addresses of NM managed interfaces, to avoid NM creating
             # new, non netplan-* connection profiles, using the existing IPs.
-            for iface in utils.nm_interfaces(restart_nm_glob, devices):
+            nm_interfaces = utils.nm_interfaces(restart_nm_glob, devices)
+            for iface in nm_interfaces:
                 utils.ip_addr_flush(iface)
             # clear NM state, especially the [device].managed=true config, as that might have been
             # re-set via an udev rule setting "NM_UNMANAGED=1"
             shutil.rmtree('/run/NetworkManager/devices', ignore_errors=True)
             utils.systemctl_network_manager('start', sync=sync)
+
+            # If 'lo' is in the nm_interfaces set we flushed it's IPs (see above) and disconnected it.
+            # NM will not bring it back automatically after restarting and we need to do that manually.
+            # For that, we need NM up and ready to accept commands
+            if 'lo' in nm_interfaces:
+                sync = True
+
             if sync:
                 # 'nmcli' could be /usr/bin/nmcli or
                 # /snap/bin/nmcli -> /snap/bin/network-manager.nmcli
@@ -309,22 +313,12 @@ class NetplanApply(utils.NetplanCommand):
                         break
                     time.sleep(0.5)
 
-    @staticmethod
-    def is_composite_member(composites, phy):
-        """
-        Is this physical interface a member of a 'composite' virtual
-        interface? (bond, bridge)
-        """
-        for composite in composites:
-            for _, settings in composite.items():
-                if not type(settings) is dict:
-                    continue
-                members = settings.get('interfaces', [])
-                for iface in members:
-                    if iface == phy:
-                        return True
-
-        return False
+            # If "lo" is managed by NM through Netplan, apply will flush its addresses and disconnect it.
+            # NM will not bring it back automatically.
+            # This is a possible scenario with netplan-everywhere. If a user tries to change the 'lo'
+            # connection with nmcli for example, NM will create a persistent nmconnection file and emit a YAML for it.
+            if 'lo' in nm_interfaces and loopback_connection:
+                utils.nm_bring_interface_up(loopback_connection)
 
     @staticmethod
     def clear_virtual_links(prev_links, curr_links, devices=[]):
@@ -360,7 +354,6 @@ class NetplanApply(utils.NetplanCommand):
         """
 
         changes = {}
-        composite_interfaces = [config_manager.bridges, config_manager.bonds]
 
         # Find physical interfaces which need a rename
         # But do not rename virtual interfaces
@@ -370,11 +363,6 @@ class NetplanApply(utils.NetplanCommand):
                 continue  # Skip if no new name needs to be set
             if not netdef._has_match:
                 continue  # Skip if no match for current name is given
-            if NetplanApply.is_composite_member(composite_interfaces, netdef.id):
-                logging.debug('Skipping composite member {}'.format(netdef.id))
-                # do not rename members of virtual devices. MAC addresses
-                # may be the same for all interface members.
-                continue
             # Find current name of the interface, according to match conditions and globs (name, mac, driver)
             current_iface_name = utils.find_matching_iface(interfaces, netdef)
             if not current_iface_name:
@@ -415,3 +403,5 @@ class NetplanApply(utils.NetplanCommand):
                 sys.exit(1)
         except OvsDbServerNotRunning as e:
             logging.warning('Cannot call Open vSwitch: {}.'.format(e))
+        except OvsDbServerNotInstalled as e:
+            logging.debug('Cannot call Open vSwitch: %s.', e)
